import { globalLookup } from "@/hook/global";
import { time2cursorX, SystemPivot } from "../pivot/cursor";
import { sidebarSelectedWires } from "@/hook/sidebar-select-wire";
import { updateWireCurrentValue } from "@/hook/utils";
import { createPivot, currentPivotId, orderedTimes, UserPivotCtxShows, UserPivots } from "../pivot/pivot-view";


/**
 * @description 将 某时间点 移动到指定位置
 * @param {number} time 需要移动到的时间点
 * @param {number} leftMargin 移动后的该时间点距离左侧多少px
 * @param {Pstate} pstate
 */
export function moveto(time, leftMargin, pstate) {
    SystemPivot.show = true;
    SystemPivot.currentTime = time;

    const { width, xOffset, xScale, tgcd, timescale } = pstate;
    let nextOffsetX =  leftMargin - (time / tgcd) * xScale;
    const sidebarWidth = pstate.sidebarWidth;

    if (nextOffsetX >= sidebarWidth) {
        nextOffsetX = sidebarWidth - 20;
    }

    pstate.oldXOffset = pstate.xOffset;
    pstate.xOffset = nextOffsetX;
    
    // 改变静态信标
    SystemPivot.updateLeft();
    SystemPivot.updateLabel(pstate.timescale);

    // 更新所有轨道的数字
    updateWireCurrentValue(time);

    // 更换渲染视图
    globalLookup.render();
}

export function toBegin() {
    const pstate = globalLookup.pstate;
    if (pstate) {
        moveto(0, pstate.sidebarWidth - 20, pstate);
    }
}

export function toEnd() {
    const pstate = globalLookup.pstate;
    if (pstate) {
        moveto(globalLookup.time, pstate.width - pstate.sidebarWidth, pstate);
    }
}

function clip(v, min, max) {
    if (v < min) {
        v = min;
    }
    if (v > max) {
        v = max;
    }
    return v;
}

export function toPrevChange() {
    const link = sidebarSelectedWires.lastLink;
    const pstate = globalLookup.pstate;

    if (link === undefined || pstate === undefined) {
        return;
    }

    const chango = globalLookup.chango[link];
    if (chango && chango.wave && chango.wave.length > 0) {
        const wave = chango.wave;

        // 如果长度为 1，直接到前面或者后面即可
        if (wave.length === 1) {
            toBegin();
        } else {
            const currentT = SystemPivot.currentTime;
            let pivot = bisearch(wave, currentT);

            const pwave = wave[pivot];
            if (pwave[0] === currentT) {
                pivot = clip(pivot - 1, 0, wave.length - 1);
            } else {
                pivot = clip(pivot, 0, wave.length - 1);
            }
            
            const targetT = wave[pivot][0];
            moveto(targetT, (pstate.width + pstate.sidebarWidth) / 2, pstate);
        }
    }
}


export function toNextChange() {
    const link = sidebarSelectedWires.lastLink;
    const pstate = globalLookup.pstate;

    if (link === undefined || pstate === undefined) {
        return;
    }

    const chango = globalLookup.chango[link];
    if (chango && chango.wave && chango.wave.length > 0) {
        const wave = chango.wave;

        // 如果长度为 1，直接到前面或者后面即可
        if (wave.length === 1) {
            toEnd();
        } else {
            const currentT = SystemPivot.currentTime;
            let pivot = bisearch(wave, currentT);
            let targetT;
            if (pivot === wave.length - 1) {
                pivot = clip(pivot + 1, 0, wave.length - 1);
                targetT = globalLookup.time;
            } else {
                pivot = clip(pivot + 1, 0, wave.length - 1);
                targetT = wave[pivot][0];
            }

            moveto(targetT, (pstate.width + pstate.sidebarWidth) / 2, pstate);
        }
    }
}

const locationManager = {
    /**
     * @description 记录所有的信标
     * @type {Map<number, HTMLElement>} 时间点到元素的映射
     */
    pivots: new Map(),

};


export function makePivot() {
    const currentT = SystemPivot.currentTime;
    // 在 currentT 创建新的信标
    createPivot(currentT);
}

export function clearAllPivot() {
    orderedTimes.length === 0;
    UserPivots.clear();
    currentPivotId.value = 0;
    UserPivotCtxShows.clear();
}


/**
 * @description 二分查找找到最近的索引下标
 * @param {number[][]} wave 
 * @param {number} time
 * @return {number} 
 */
export function bisearch(wave, time) {
    const times = wave.map(p => p[0]);

    // 二分查找，并将结果存入 i
    let i = 0, j = wave.length - 1;
    while (i < j) {
        if (times[i] === time) {
            break;
        }
        if (times[j] <= time) {
            i = j;
            break;
        }
        if (j - i === 1) {
            break;
        }
        const mid = (i + j) >> 1;
        if (times[mid] > time) {
            j = mid;
        } else {
            i = mid;
        }
    }

    return i;
}