
/**
 * @description 搜索数值的每一个结果的数据结构
 * @typedef {Object} toolbarSearchResultItem
 * @property {string} name
 * @property {string} link
 * @property {number} width
 * @property {string} type
 * @property {number} value
 */

import { globalLookup } from "@/hook/global";
import { sidebarSelectedWires } from "@/hook/sidebar-select-wire";
import { reactive, ref } from "vue";
import { bisearch, moveto } from "./cursor-location";
import { SystemPivot } from "../pivot/cursor";
import { FormatValueRender, JSValueRender } from "@/hook/wave-view/toolbar/renderFormat";
import { ElMessage } from "element-plus";
import i18n from '@/i18n/index';

const { t } = i18n.global;

export const searchMode = ref(1);
export const valueSearchText = ref(t('toolbar.search.value.searching'));

export const toolbarValueSearch = reactive({
    content: '',
    displayResult: false,
    mouseOnResult: false,

    /**
     * @type {toolbarSearchResultItem[]}
     */
    searchResult: [],

    /**
     * @type {Set<string>}
     */
    resultLinkSet: new Set(),

    /**
     * @description 当前选择的索引
     */
    cursorIndex: -1
});


function toPrevValue() {
    // TODO: 空间换时间，提升性能
    const prevTimes = [];

    const currentT = SystemPivot.currentTime;
    // SystemPivot.updateLabel(currentT);
    // SystemPivot.updateLeft();

    const searchContent = toolbarValueSearch.content.trim();

    for (const link of globalLookup.sidebarSelectedWireLinks) {
        const chango = globalLookup.chango[link];
        const signal = globalLookup.link2CurrentWires.get(link);
        const valueRender = new FormatValueRender(link, signal.size, '');
        const { wave } = chango;
        
        let i = bisearch(wave, currentT);
        console.log(i);
        
        // 往前寻找
        while (i >= 0) {
            const [t, val, mask] = wave[i];
            const valString = mask === 1 ? 'x' : valueRender.render(val, -1);
            console.log(val, valString, wave[i]);
            
            if (valString === searchContent && t != currentT) {
                break;
            }
            i --;
        }

        if (i >= 0 && wave[i][0] < currentT) {
            prevTimes.push(wave[i][0]);
        }
    }

    if (prevTimes.length > 0) {        
        const preT = Math.max(...prevTimes);
        const pstate = globalLookup.pstate;
        moveto(preT, (pstate.width + pstate.sidebarWidth) / 2, pstate);
    } else {
        valueSearchText.value = t('toolbar.search.value.already-to-head');
    }
}

function toNextValue() {
    const prevTimes = [];
    
    const currentT = SystemPivot.currentTime;
    // SystemPivot.updateLabel(currentT);
    // SystemPivot.updateLeft();

    const searchContent = toolbarValueSearch.content.trim();

    for (const link of globalLookup.sidebarSelectedWireLinks) {
        const chango = globalLookup.chango[link];
        const signal = globalLookup.link2CurrentWires.get(link);
        const valueRender = new FormatValueRender(link, signal.size, '');
        const { wave } = chango;
        
        let i = bisearch(wave, currentT);
        const length = wave.length;
        // 往后寻找
        while (i < length) {
            const [t, val, mask] = wave[i];
            const valString = mask === 1 ? 'x' : valueRender.render(val, -1);
            console.log(val, mask, valString);
            
            if (valString === searchContent && t != currentT) {
                break;
            }
            i ++;
        }

        if (i < length && wave[i][0] > currentT) {
            prevTimes.push(wave[i][0]);
        }
    }

    if (prevTimes.length > 0) {
        const preT = Math.min(...prevTimes);
        const pstate = globalLookup.pstate;
        moveto(preT, (pstate.width + pstate.sidebarWidth) / 2, pstate);
    } else {
        valueSearchText.value = t('toolbar.search.value.already-to-tail');
    }
}


function toPrevName() {
    // 清空
    sidebarSelectedWires.clear();
    const currentIndex = toolbarValueSearch.cursorIndex;
    const searchResultNum = toolbarValueSearch.searchResult.length;
    const targetIndex = (currentIndex - 1 + searchResultNum) % searchResultNum;
    if (targetIndex !== currentIndex) {
        toolbarValueSearch.cursorIndex = targetIndex;
        const { link } = toolbarValueSearch.searchResult[targetIndex];
        sidebarSelectedWires.add(link);
    }
}

function toNextName() {
    // 清空
    sidebarSelectedWires.clear();
    const currentIndex = toolbarValueSearch.cursorIndex;
    const searchResultNum = toolbarValueSearch.searchResult.length;
    const targetIndex = (currentIndex + 1) % searchResultNum;    
    if (targetIndex !== currentIndex) {
        toolbarValueSearch.cursorIndex = targetIndex;
        const { link } = toolbarValueSearch.searchResult[targetIndex];
        sidebarSelectedWires.add(link);
    }
}



export function onClickPrev() {
    if (globalLookup.sidebarSelectedWireLinks.size === 0 || toolbarValueSearch.content.trim().length === 0) {
        return;
    }

    switch (searchMode.value) {
        case 0:
            toPrevValue();
            break;
        case 1:
            toPrevName();
            break;

        default:
            break;
    }
}

export function onClickNext() {
    if (globalLookup.sidebarSelectedWireLinks.size === 0 || toolbarValueSearch.content.trim().length === 0) {
        return;
    }

    switch (searchMode.value) {
        case 0:
            toNextValue();
            break;
        case 1:
            toNextName();
            break;

        default:
            break;
    }
}