
const platform = navigator.platform;

/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<EventHandlerKind, Handler>} eventHandler 
 * @param {KeyboardEvent} event 
 */
function windowsKeydown(element, pstate, deso, eventHandler, event) {
    if (event.ctrlKey) {
        if (event.key === '=') {
            eventHandler['scaleUp'].fn(pstate);
            deso.render();
            event.preventDefault();
        } else if (event.key === '-') {
            eventHandler['scaleDown'].fn(pstate);
            deso.render();
            event.preventDefault();
        }
    }

    if (event.shiftKey) {
        switch (event.key) {
        case 'ArrowLeft':
            eventHandler['scrollLeft'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowRight':
            eventHandler['scrollRight'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowUp':
            eventHandler['scrollUp'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowDown':
            eventHandler['scrollDown'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        default:
            break;
        }
    }

    // event.preventDefault();
}

/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<EventHandlerKind, Handler>} eventHandler 
 * @param {KeyboardEvent} event 
 */
function linuxKeydown(element, pstate, deso, eventHandler, event) {
    if (event.ctrlKey) {
        if (event.key === '=') {
            eventHandler['scaleUp'].fn(pstate);
            deso.render();
            event.preventDefault();
        } else if (event.key === '-') {
            eventHandler['scaleDown'].fn(pstate);
            deso.render();
            event.preventDefault();
        }
    }

    if (event.shiftKey) {
        switch (event.key) {
        case 'ArrowLeft':
            eventHandler['scrollLeft'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowRight':
            eventHandler['scrollRight'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowUp':
            eventHandler['scrollUp'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowDown':
            eventHandler['scrollDown'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        default:
            break;
        }
    }
    // event.preventDefault();
}

/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<EventHandlerKind, Handler>} eventHandler 
 * @param {KeyboardEvent} event 
 */
function macOsKeydown(element, pstate, deso, eventHandler, event) {
    if (event.metaKey) {
        if (event.key === '=') {
            eventHandler['scaleUp'].fn(pstate);
            deso.render();
            event.preventDefault();
        } else if (event.key === '-') {
            eventHandler['scaleDown'].fn(pstate);
            deso.render();
            event.preventDefault();
        }
    }

    if (event.shiftKey) {
        switch (event.key) {
        case 'ArrowLeft':
            eventHandler['scrollLeft'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowRight':
            eventHandler['scrollRight'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowUp':
            eventHandler['scrollUp'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        case 'ArrowDown':
            eventHandler['scrollDown'].fn(pstate);
            deso.render();
            event.preventDefault();
            break;
        default:
            break;
        }
    }
    // event.preventDefault();
}

/**
 * 
 * @param {WaveElements} element 
 * @param {Pstate} pstate 
 * @param {Deso} deso 
 * @param {Record<string, Handler>} eventHandler 
 * @returns {(event: KeyboardEvent) => void}
 */
function registerKeyEvent(element, pstate, deso, eventHandler) {
    const platform = navigator.platform;
    if (platform.startsWith('Win')) {
        return event => windowsKeydown(element, pstate, deso, eventHandler, event);
    } else if (platform.startsWith('Linux')) {
        return event => linuxKeydown(element, pstate, deso, eventHandler, event);
    } else if (platform.startsWith('Mac')) {
        return event => macOsKeydown(element, pstate, deso, eventHandler, event);
    } else {
        throw Error('不支持的操作系统！');
    }
}


export {
    registerKeyEvent
};