function getRatio() {
    var ratio=0;
    var screen=window.screen;
    var ua=navigator.userAgent.toLowerCase();
    if(window.devicePixelRatio !== undefined) {
        ratio=window.devicePixelRatio;    
    }
    else if(~ua.indexOf('msie')) {
        if(screen.deviceXDPI && screen.logicalXDPI) {
            ratio=screen.deviceXDPI/screen.logicalXDPI;        
        }
    
    }
    else if(window.outerWidth !== undefined && window.innerWidth !== undefined) {
        ratio=window.outerWidth/window.innerWidth;
    }

    if(ratio) {
        ratio=Math.round(ratio*100);    
    }
    return ratio;
}

const screenHeightPixel = window.screen.height * getRatio() / 100;
const screenWidthPixel = window.screen.width * getRatio() / 100;

// rgba 颜色通道，都是预设的颜色
export const gl_Colors_template = [
    [0, 0, 0, 0 ],       // 0: 空
    [0, 0, 255, 1],      // 1: 未知态 X                         默认颜色
    [51, 230, 26, 1],    // 2: value = 0 用于 width = 1 的信号  默认颜色
    [51, 230, 26, 1],    // 3: value = 1 用于 width = 1 的信号  默认颜色
    [230, 51, 51, 1],    // 4: 高阻态 Z                        默认颜色
    [124, 77, 255, 1],   // 5: vec 用于 width > 1 的信号
    [255, 0, 255, 1],    // 6: yellow
    [255, 0, 255, 1],    // 7: strange purple
    [0, 255, 0, 0.5],    // 8: (l L) weak 0
    [255, 0, 255, 0.5],  // 9: (h H) weak 1
    [255, 0, 0, 0.5],    // 10: (w W) weak unknown

    // >= 11 自定义颜色
    [255, 140, 0, 1],
    [255, 140, 0, 1],
    [144, 238, 144, 1],
    [0, 206, 209, 1],
    [30, 144, 255, 1],
    [199, 21, 133, 1],
    [128, 31, 255, 1],
    [124, 165, 50, 1]
];

function makeGlColors() {
    const glcolors = [];
    for (const rgba of gl_Colors_template) {
        const r = rgba[0] / 255;
        const g = rgba[1] / 255;
        const b = rgba[2] / 255;
        const a = rgba[3];
        glcolors.push(r, g, b, a);
    }
    // 制作 mask
    for (const rgba of gl_Colors_template) {
        const r = rgba[0] / 255;
        const g = rgba[1] / 255;
        const b = rgba[2] / 255;
        glcolors.push(r, g, b, 0.2);
    }
    return glcolors;
}


// 根据 gl_Colors_template 计算出 gl_Colors
// 控制颜色 0 - 1 归一化的 bgr
const gl_Colors = new Float32Array(makeGlColors());

export let maskColorIndexOffset = gl_Colors_template.length;


// 特殊偏移量
// 第一列 Y 第二列 保留
const gl_Shifts = new Float32Array([ // 14
    0,  0,  // 0
    1,  0,  // 1
    1,  0,  // 2
    1,  0,  // 3
    -1, 0,  // 4
    -1, 0,  // 5
    -1, 0   // 6
]);

// 视屏幕本身而定
const posYFactor = 10000;
const glslInputLength = 5;

// 为了满足 vec 类型的波形的开头和结尾的那个小小的内嵌的、不受scale影响的值
// 第一列 Y 第二列 X 在所有计算完成后的额外偏移量
const barShift = 0.004;

const gl_Shifts_for_bar = new Float32Array([
    0,         0,  // 0
    barShift,  0,  // 1
    -barShift, 0,  // 2
    0,         0,  // 3
    barShift,  0, // 4
    -barShift, 0, // 5
    0,         0,   // 6
]);


const lineWidth = 0.004 * 3800 / screenWidthPixel;            // 不能写为 0.0045 这样会因为插值造成不同线段的宽度不一致的问题
const widthShift = 0.002;

const gl_WidthShifts = new Float32Array([
    0, widthShift,                  // 0
    - widthShift, widthShift,       // 1
    - widthShift, 0,                // 2
    - widthShift, - widthShift,     // 3
    0, - widthShift,                // 4
    widthShift, - widthShift,       // 5
    widthShift, 0,                  // 6
    widthShift, widthShift,         // 7
    0, 0                            // 8
]);


// TODO: 之前的波形的竖向线条的宽度在缩放式有时会变窄。一直不一致，不知道是为什么，
// 我认为是glsl内部精度损失的问题，稍微把横向的宽度拉大后，可以大幅度改善
// 如果未来有时间，搞清楚这里是怎么回事。复现方法：把 wsMdFactor 修改为 1
const wsMdFactorX = 1.2;
const wsMdFactorY = 1.3;
const ladderAnalog_GL_WidthShifts = new Float32Array([
    0, widthShift * wsMdFactorY,                  // 0
    - widthShift * wsMdFactorX, widthShift * wsMdFactorY,       // 1
    - widthShift * wsMdFactorX, 0,                // 2
    - widthShift * wsMdFactorX, - widthShift * wsMdFactorY,     // 3
    0, - widthShift * wsMdFactorY,                // 4
    widthShift * wsMdFactorX, - widthShift * wsMdFactorY,       // 5
    widthShift * wsMdFactorX, 0,                  // 6
    widthShift * wsMdFactorX, widthShift * wsMdFactorY          // 7
]);


const lineAnalog_WidthShift = 0.0015;
const lineAnlog_GL_WidthShifts = new Float32Array([
    0, lineAnalog_WidthShift,                  // 0
    - lineAnalog_WidthShift, lineAnalog_WidthShift,       // 1
    - lineAnalog_WidthShift, 0,                // 2
    - lineAnalog_WidthShift, - lineAnalog_WidthShift,     // 3
    0, - lineAnalog_WidthShift,                // 4
    lineAnalog_WidthShift, - lineAnalog_WidthShift,       // 5
    lineAnalog_WidthShift, 0,                  // 6
    lineAnalog_WidthShift, lineAnalog_WidthShift,         // 7
    0, 0                            // 8
]);


function prettyPrint(array) {
    const stack = [];
    for (const num of array) {
        if (stack.length === 5) {
            console.log(stack);
            stack.length = 0;
        }
        stack.push(num);
    }
    if (stack.length > 0) {
        console.log(stack);
    }
}

export {
    getRatio,
    gl_Colors,
    gl_Shifts,
    posYFactor,
    lineWidth,
    widthShift,
    gl_WidthShifts,
    screenWidthPixel,
    screenHeightPixel,
    gl_Shifts_for_bar,
    lineAnlog_GL_WidthShifts,
    barShift,
    glslInputLength,
    prettyPrint,
    lineAnalog_WidthShift,
    ladderAnalog_GL_WidthShifts
};